-----------------------------------------------------------------------------------------------
-- Client Lua Script for MJPriority
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "Apollo"
require "GameLib"
require "Unit"
require "AbilityBook"

-----------------------------------------------------------------------------------------------
-- MJPriority Module Definition
-----------------------------------------------------------------------------------------------
local MJPriority = {} 
 
MJPriority.PriorityFrame = {}
MJPriority.Settings = {
	size = 80,
	Priorities = {},
	Prioritynames = {
		prio1 = "Ruin",
		prio2 = "Punish",
		prio3 = "Impale",
		prio4 = "Shred",
		prio5 = "",
		prio6 = "",
		prio7 = "",
		prio8 = "",
	}
}
MJPriority.CDs = {}
MJPriority.selectPrio = {}
MJPriority.RuinIDs = {
	Ruin  = 38841,
	Ruin1 = 59499,
	Ruin2 = 59500,
	Ruin3 = 59501,
	Ruin4 = 59502,
	Ruin5 = 59503,
	Ruin6 = 59504,
	Ruin7 = 59505,
	Ruin8 = 59506
}
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function MJPriority:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	return o
end

function MJPriority:Init()
	local bHasConfigureFunction = false	
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	
end
 

-----------------------------------------------------------------------------------------------
-- MJPriority OnLoad
-----------------------------------------------------------------------------------------------
function MJPriority:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("MJPriority.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)	
end

-----------------------------------------------------------------------------------------------
-- MJPriority OnDocLoaded
-----------------------------------------------------------------------------------------------
function MJPriority:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "MJPriorityForm", nil, self)
		self.confWnd = Apollo.LoadForm(self.xmlDoc, "MJPriorityOptions", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    --self:OnMJPriorityOn()
		--self.confWnd:Show(false)
		--self.wndMain:Show(true)
		
		Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
		Apollo.RegisterEventHandler("AbilityBookChange", "OnAbilityBookChange", self)

		Apollo.RegisterSlashCommand("mjp", "OnMJPriorityOn", self)
		Apollo.RegisterSlashCommand("mjpconfig", "OnMjpconfig", self)
		Apollo.RegisterSlashCommand("printbuffs", "printBuffs", self)
		Apollo.RegisterSlashCommand("printabilities", "printAbilities", self)
		
		Apollo.RegisterTimerHandler("LASChangeTimer", "OnLASChange", self)
		
		-- load priorityframes.. maybe pretty useless.			
		self.PriorityFrame["Punish"] = self.wndMain:FindChild("Punish")
		self.PriorityFrame["Punish"]:Show(false, true)
		self.PriorityFrame["Impale"] = self.wndMain:FindChild("Impale") 
		self.PriorityFrame["Impale"]:Show(false, true)
		self.PriorityFrame["Ruin"] = self.wndMain:FindChild("Ruin") 
		self.PriorityFrame["Ruin"]:Show(false, true)
		self.PriorityFrame["Concussive Kicks"] = self.wndMain:FindChild("ConcussiveKicks") 
		self.PriorityFrame["Concussive Kicks"]:Show(false, true)
		self.PriorityFrame["Shred"] = self.wndMain:FindChild("Shred") 
		self.PriorityFrame["Shred"]:Show(false, true)
		self.PriorityFrame["Neutralize"] = self.wndMain:FindChild("Neutralize") 
		self.PriorityFrame["Neutralize"]:Show(false, true)
		self.PriorityFrame["Analyze Weakness"] = self.wndMain:FindChild("AnalyzeWeakness") 
		self.PriorityFrame["Analyze Weakness"]:Show(false, true)
		
		--restore settings
		self:setSize(self.Settings["size"])
		self.confWnd:FindChild("SliderWidth"):SetValue(self.Settings["size"])
		self:OnAbilityBookChange()
		self:OnRequiredFlagsChanged()
	end
end


function MJPriority:OnRequiredFlagsChanged()
	if GameLib.GetPlayerUnit() then
		self:OnCharacterCreated()
	else
		Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
	end
end

function MJPriority:OnCharacterCreated()
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer:GetClassId() ~= GameLib.CodeEnumClass.Stalker then
		return
	end
	Apollo.RegisterEventHandler("VarChange_FrameCount", "OnFrame", self)
end

--creat LASChangeTimer to delay searching through new LAS, otherwise it bugs out (too slow or... dunno :<)
function MJPriority:OnAbilityBookChange()
	Apollo.CreateTimer("LASChangeTimer", 0.2, false)
end

--if LAS changes, clear all priorities and CDs and fill them again
function MJPriority:OnLASChange()
	self.Settings.Priorities = {}
	self.CDs = {}
	self:SetAllPriorities()
	
	local abilities = AbilityBook.GetAbilitiesList()
	local las = ActionSetLib.GetCurrentActionSet()
	
	if las ~= nil then
		self.wndMain:FindChild("Preparation"):Show(false)
		self.wndMain:FindChild("Collapse"):Show(false)
		self.wndMain:FindChild("Stagger"):Show(false)
		self.wndMain:FindChild("FalseRetreat"):Show(false)
		self.wndMain:FindChild("Pounce"):Show(false)
		self.wndMain:FindChild("TacticalRetreat"):Show(false)
	
	
		for number, id in pairs(las) do
			for _, ability in pairs(abilities) do
				if ability.strName == "Preparation" then
					if ability.nId == id then
						self.wndMain:FindChild("Preparation"):Show(true)
						self.CDs.Preparation = ability.tTiers[ability.nCurrentTier].splObject
					end
				end
				if ability.strName == "Collapse" then
					if ability.nId == id then
						self.wndMain:FindChild("Collapse"):Show(true)
						self.CDs.Collapse = ability.tTiers[ability.nCurrentTier].splObject
					end
				end
				if ability.strName == "Stagger" then
					if ability.nId == id then
						self.wndMain:FindChild("Stagger"):Show(true)
						self.CDs.Stagger = ability.tTiers[ability.nCurrentTier].splObject
					end
				end
				if ability.strName == "False Retreat" then
					if ability.nId == id then
						self.wndMain:FindChild("FalseRetreat"):Show(true)
						self.CDs.FalseRetreat = ability.tTiers[ability.nCurrentTier].splObject
					end
				end
				if ability.strName == "Pounce" then
					if ability.nId == id then
						self.wndMain:FindChild("Pounce"):Show(true)
						self.CDs.Pounce = ability.tTiers[ability.nCurrentTier].splObject
					end
				end
				if ability.strName == "Tactical Retreat" then
					if ability.nId == id then
						self.wndMain:FindChild("TacticalRetreat"):Show(true)
						self.CDs.TacticalRetreat = ability.tTiers[ability.nCurrentTier].splObject
					end
				end
			end
		end
	end
end

-- /mjp activates/deactivates priority window
function MJPriority:OnMJPriorityOn()
	if self.wndMain:IsShown() then
		self.wndMain:Show(false)
	else
		self.wndMain:Show(true)
	end
end
	
-- on each frame, check all CDs and look for next Priority
function MJPriority:OnFrame()
	self:checkCDs()
				
	local nextPrio = self:checkPrio()
	for key, value in pairs(self.PriorityFrame) do
		self.PriorityFrame[key]:Show(false)
	end
	
	self.PriorityFrame[nextPrio]:Show(true)
end

--check if stealthed
function MJPriority:checkStealth()
	local player = GameLib.GetPlayerUnit()
	for _, activeBuff in pairs(player:GetBuffs().arBeneficial) do
	    if activeBuff.splEffect:GetId() == 38784 then
			return true
		end
   	end
	return false
end

--check if spell is in LAS [not needed anymore, but left for future uses]
--function MJPriority:checkLAS(spellid)
--	local las = ActionSetLib.GetCurrentActionSet()
--	if las ~= nil then
--		for number, id in pairs(las) do
--			if id == spellid then
--				return true
--			end
--		end
--	end
--	return false
--end

--check if given spell can/should be used. returns true/false
function MJPriority:checkSpell(spell)
	local player = GameLib.GetPlayerUnit()
	local target = player:GetTarget()
	
	if spell:GetCooldownRemaining() ~= 0 then
		return false
	end
		
	-- check for each spell
	if spell:GetName() == "Punish" then
		for _, activeBuff in pairs(player:GetBuffs().arBeneficial) do
	    	if activeBuff.splEffect:GetId() == 70775 then
				-- maybe change value, < 30 is a safe bet for now with human lag, but maybe not optimal.
				if player:GetResource(3) < 30 then
					return true
				end
			end
   		end
	end
	
	if spell:GetName()== "Impale" then
		if spell:GetId() == 59362 then
			if player:GetResource(3) > 30 then
				return true
			end
		else
			if player:GetResource(3) > 35 then
				return true
			end
		end
	end
	
	if spell:GetName()== "Neutralize" then
		for _, activeBuff in pairs(player:GetBuffs().arBeneficial) do
	    	if activeBuff.splEffect:GetId() == 58664 then
				if player:GetResource(3) > (15 + (5 * activeBuff["nCount"])) then
					return true
				else
					return false
				end
			end
   		end
		if player:GetResource(3) > 15 then
			return true
		end
	end	
	
	if spell:GetName()== "Analyze Weakness" then
		if player:GetResource(3) > 15 then
			return true
		end
	end
	
	if spell:GetName()== "Concussive Kicks" then
		if player:GetResource(3) > 15 then
			return true
		end
	end
	
	if spell:GetName()== "Ruin" then
		if target ~= nil then
			for _, activeBuff in pairs(target:GetBuffs().arHarmful) do
				for _, RuinID in pairs(self.RuinIDs) do
					if activeBuff.splEffect:GetId() == RuinID then
						if activeBuff["fTimeRemaining"] > 1.5 then
							return false
						end
					end
				end
			end
		end
		return true
	end
	
	if spell:GetName()== "Shred" then
		return true
	end
	
	return false
	
end		

--check for next priority, returns spell in string
function MJPriority:checkPrio()

	if self:checkStealth() then
		return "Impale"
	end
	
	for i=1,8 do
		if self.Settings["Priorities"]["prio" .. i] ~= nil then
			if self:checkSpell(self.Settings["Priorities"]["prio" .. i]) then
				return self.Settings["Priorities"]["prio" .. i]:GetName()
			end
		end
	end

	--Print("checkPrio not working correctly!")
	return "Shred"
end

function MJPriority:checkNextPrio()
	--maybe later
end

function MJPriority:checkCDs()
	local gadget = GameLib.GetGadgetAbility()
	local innate = GameLib.GetCurrentClassInnateAbilitySpell()
		
	if self.CDs.Preparation ~= nil then
		if self.CDs.Preparation:GetCooldownRemaining() ~= 0 then
			self.wndMain:FindChild("Preparation"):SetText(math.floor(self.CDs.Preparation:GetCooldownRemaining()))
			self.wndMain:FindChild("Preparation"):SetStyle("Picture", false)
		else
			self.wndMain:FindChild("Preparation"):SetStyle("Picture", true)
			self.wndMain:FindChild("Preparation"):SetText("")
		end
	end
	
	if self.CDs.Collapse ~= nil then
		if self.CDs.Collapse:GetCooldownRemaining() ~= 0 then
			self.wndMain:FindChild("Collapse"):SetText(math.floor(self.CDs.Collapse:GetCooldownRemaining()))
			self.wndMain:FindChild("Collapse"):SetStyle("Picture", false)
		else
			self.wndMain:FindChild("Collapse"):SetStyle("Picture", true)
			self.wndMain:FindChild("Collapse"):SetText("")
		end
	end
	
	if self.CDs.Pounce ~= nil then
		if self.CDs.Pounce:GetCooldownRemaining() ~= 0 then
			self.wndMain:FindChild("Pounce"):SetText(math.floor(self.CDs.Pounce:GetCooldownRemaining()))
			self.wndMain:FindChild("Pounce"):SetStyle("Picture", false)
		else
			self.wndMain:FindChild("Pounce"):SetStyle("Picture", true)
			self.wndMain:FindChild("Pounce"):SetText("")
		end
	end
	
	if self.CDs.Stagger ~= nil then
		if self.CDs.Stagger:GetCooldownRemaining() ~= 0 then
			self.wndMain:FindChild("Stagger"):SetText(math.floor(self.CDs.Stagger:GetCooldownRemaining()))
			self.wndMain:FindChild("Stagger"):SetStyle("Picture", false)
		else
			self.wndMain:FindChild("Stagger"):SetStyle("Picture", true)
			self.wndMain:FindChild("Stagger"):SetText("")
		end
	end
	
	if self.CDs.FalseRetreat ~= nil then
		if self.CDs.FalseRetreat:GetCooldownRemaining() ~= 0 then
			self.wndMain:FindChild("FalseRetreat"):SetText(math.floor(self.CDs.FalseRetreat:GetCooldownRemaining()))
			self.wndMain:FindChild("FalseRetreat"):SetStyle("Picture", false)
		else
			self.wndMain:FindChild("FalseRetreat"):SetStyle("Picture", true)
			self.wndMain:FindChild("FalseRetreat"):SetText("")
		end
	end
	
	if self.CDs.TacticalRetreat ~= nil then
		if self.CDs.TacticalRetreat:GetCooldownRemaining() ~= 0 then
			self.wndMain:FindChild("TacticalRetreat"):SetText(math.floor(self.CDs.TacticalRetreat:GetCooldownRemaining()))
			self.wndMain:FindChild("TacticalRetreat"):SetStyle("Picture", false)
		else
			self.wndMain:FindChild("TacticalRetreat"):SetStyle("Picture", true)
			self.wndMain:FindChild("TacticalRetreat"):SetText("")
		end
	end
	
		
	if gadget ~= nil then			
		if gadget:GetCooldownRemaining() ~= 0 then
			self.wndMain:FindChild("Power"):SetText(math.floor(gadget:GetCooldownRemaining()))
			self.wndMain:FindChild("Power"):SetStyle("Picture", false)
		else
			self.wndMain:FindChild("Power"):SetStyle("Picture", true)
			self.wndMain:FindChild("Power"):SetText("")
		end
	end

	if innate ~= nil then			
		if innate:GetCooldownRemaining() ~= 0 then
			self.wndMain:FindChild("NanoSkin"):SetText(math.floor(innate:GetCooldownRemaining()))
			self.wndMain:FindChild("NanoSkin"):SetStyle("Picture", false)
		else
			self.wndMain:FindChild("NanoSkin"):SetStyle("Picture", true)
			self.wndMain:FindChild("NanoSkin"):SetText("")
		end
	end
end

function MJPriority:setPrio(pos, spell)
	local abilities = AbilityBook.GetAbilitiesList()
	local las = ActionSetLib.GetCurrentActionSet()
	
	if las ~= nil then
		for number, id in pairs(las) do
			for _, ability in pairs(abilities) do
				if ability.strName == spell then
					if ability.nId == id then
						self.Settings.Priorities[pos] = ability.tTiers[ability.nCurrentTier].splObject
					end
				end
			end
		end
	end
	self.Settings.Prioritynames[pos] = spell
	self.confWnd:FindChild(pos):SetText(spell)
end

function MJPriority:SetAllPriorities()
	MJPriority.Settings.Priorities = {}
 	for key, value in pairs(self.Settings.Prioritynames) do
		self:setPrio(key, value)
	end
end


function MJPriority:OnMjpconfig()
	for pos, ability in pairs(self.Settings.Prioritynames) do
		self.confWnd:FindChild(pos):SetText(ability)
	end
		
	self.confWnd:Show(true)
end

function MJPriority:printBuffs()
	local player = GameLib.GetPlayerUnit()
	local target = player:GetTarget()
	Print(player:GetClassId())
	if player ~= nil then
		Print("Player Buffs:")
		for _, activeBuff in pairs(player:GetBuffs().arBeneficial) do
			Print(activeBuff.splEffect:GetName() .. " id: " .. activeBuff.splEffect:GetId())
		end
	end
	
	if player ~= nil then
		Print("Player Debuffs:")
		for _, activeBuff in pairs(player:GetBuffs().arHarmful) do
			Print(activeBuff.splEffect:GetName() .. " id: " .. activeBuff.splEffect:GetId())
		end
	end
		
	if target ~= nil then
		Print("Target Buffs:")
		for _, activeBuff in pairs(target:GetBuffs().arBeneficial) do
			Print(activeBuff.splEffect:GetName() .. " id: " .. activeBuff.splEffect:GetId())
		end
	end
	
	if target ~= nil then
		Print("Target Debuffs:")
		for _, activeBuff in pairs(target:GetBuffs().arHarmful) do
			Print(activeBuff.splEffect:GetName() .. " id: " .. activeBuff.splEffect:GetId())
		end
	end	
end

function MJPriority:printAbilities()
	local abilities = AbilityBook.GetAbilitiesList()
	local active = ActionSetLib.GetCurrentActionSet()
	local spec = AbilityBook.GetCurrentSpec()
	Print(spec)
	for number, id in pairs(active) do
		Print(number .. " is " .. id)
		for _, ability in pairs(abilities) do
			if id  == ability.nId then
				Print(ability.strName .. " id: " .. ability.nId)
			end
		end
	end
	
end

function MJPriority:OnWindowManagementReady()
    Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "MJPriority"})
end

--option shizzle
function MJPriority:OnSliderSizeChange(wndHandler, wndControl, fNewValue, fOldValue, bOkToChange)
	self.Settings["size"] = fNewValue
	self:setSize(fNewValue)
	wndControl:SetValue(fNewValue)
end

--save shizzle
function MJPriority:OnSave(eType)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		return self.Settings
  	end
end

function MJPriority:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		for key, value in pairs(tSavedData) do
			self.Settings[key] = value
		end
	end		
end

function MJPriority:setSize(newsize)
	for key, value in pairs(self.PriorityFrame) do
		self.PriorityFrame[key]:SetAnchorOffsets(0, 0, newsize, newsize)
	end
	
	halfsize = math.floor(newsize / 2) 
	pointfivesize = math.floor(newsize * 1.5)
	doublesize = math.floor(newsize * 2)
	
	self.wndMain:SetAnchorOffsets(0, 0, math.floor(newsize * 3.5), newsize)	
	self.wndMain:FindChild("NanoSkin"):SetAnchorOffsets(newsize, halfsize, pointfivesize, newsize)
	self.wndMain:FindChild("Power"):SetAnchorOffsets(pointfivesize, halfsize, doublesize , newsize)
	self.wndMain:FindChild("Preparation"):SetAnchorOffsets(doublesize , halfsize, math.floor(newsize * 2.5), newsize)
	self.wndMain:FindChild("Pounce"):SetAnchorOffsets(math.floor(newsize * 2.5), halfsize, math.floor(newsize * 3), newsize)
	self.wndMain:FindChild("TacticalRetreat"):SetAnchorOffsets(math.floor(newsize * 3), halfsize, math.floor(newsize * 3.5), newsize)
	
	self.wndMain:FindChild("Stagger"):SetAnchorOffsets(newsize, 0, pointfivesize , halfsize)
	self.wndMain:FindChild("Collapse"):SetAnchorOffsets(pointfivesize , 0, doublesize , halfsize)
	self.wndMain:FindChild("FalseRetreat"):SetAnchorOffsets(doublesize , 0, math.floor(newsize * 2.5), halfsize)
end



---------------------------------------------------------------------------------------------------
-- MJPriorityOptions Functions
---------------------------------------------------------------------------------------------------

function MJPriority:OnPrioButtonSignal( wndHandler, wndControl, eMouseButton )
	self.confWnd:FindChild("priorities"):Show(true)
	self.selectPrio[wndControl:GetName()] = ""
end

function MJPriority:OnPrioSpellButtonSignal( wndHandler, wndControl, eMouseButton )
	for key, _ in pairs(self.selectPrio) do
		if wndControl:GetName() ~= "None" then
			self:setPrio(key, wndControl:GetName())
		else
			self:setPrio(key, "")
		end
	end
	self.selectPrio = {}
	self.confWnd:FindChild("priorities"):Show(false)
	self:SetAllPriorities()
end

-----------------------------------------------------------------------------------------------
-- MJPriority Instance
-----------------------------------------------------------------------------------------------
local MJPriorityInst = MJPriority:new()
MJPriorityInst:Init()
